package de.uni_passau.fim.seibt.gitwrapper.repo;

import java.util.Optional;

import de.uni_passau.fim.seibt.gitwrapper.process.ProcessExecutor.ExecRes;

/**
 * A {@link MergeStrategy} representing the default 'git merge' merge implementation.
 */
public class DefaultMergeStrategy implements MergeStrategy {

    private final Repository repo;

    /**
     * The default merge strategy that git uses when no
     * <a href="https://git-scm.com/docs/git-merge#_merge_strategies>git merge</a> strategy is specified.
     *
     * @param repo
     *         the repo the strategy operates on
     */
    public DefaultMergeStrategy(Repository repo) {
        this.repo = repo;
    }

    @Override
    public Optional<Status> merge(Reference left, Reference right) {
        Optional<ExecRes> mergeRes = repo.getGit().exec(repo.getDir(), "merge", "-n", "-q", right.getId());

        if (mergeRes.isPresent() && mergeRes.get().failed()) {
            // merge failed, and we don't want to know the state of the repo at the parent
            return Optional.empty();
        }

        return mergeRes.flatMap(res -> repo.getStatus());
    }

    @Override
    public String toString() {
        return getClass().getSimpleName();
    }
}
